##SCRIPT FOR PROCESSING MCL OUTPUT FOR FEEDING TO JCE
##Created by Marc Facciotti 11/11/2015
##Edited by Andrew Yao 07/06/2020
##1. NEED GFF FILES (DERIVED FROM PARSING .GBK FILE) - MAKE SURE TO RENAME .GBK FILE WITH GENOME NAME FROM "ORGANISM" FLAG
##2. NEED THE OUTPUT FILE FROM MCL - WILL LIKELY HAVE A .mlc suffix with inflation parameter appended
##3. CREATE DIRECTORIES for (A) original GFF Files (B) new GFF Files (C) GBK files (D) MCL OUTPUT
##4. FOR THE MOMENT - RUN SCRIPT IN PARTS and VERIFY OUTPUT BEFORE PROCEEDING TO NEXT STEP
##5. YOU CAN AND SHOULD THINK ABOUT STARTING MULTIPLE INSTANCES OF STEP 3 

##STEP 1 - CREATE NEW GFFs
##SET WORKING DIRECTORY TO NEW GFFwID directory
##Read in file names - SET PATH TO original GFF Files directory here 
library(parallel)
library(purrr)

args <- commandArgs(TRUE)
cores <- detectCores()
print(cores);

(WD <- getwd())
if (!is.null(WD)) setwd(WD)
setwd("GFFwID")

##get paths of subdirectories 
GFF <- paste(WD, "/GFF", sep = "", collapse = NULL)
print (GFF)

GFFwID <- paste(WD, "/GFFwID", sep = "", collapse = NULL)
print (GFFwID)



gff.names2 <- list.files(GFF,full=FALSE)
gff.files2 <- list.files(GFF,full=TRUE)

print("Converting GFF Files....")

##READ IN EACH INDIVIDUAL OLD GFF INTO A MATRIX gffContent
for (p in 1:length(gff.names2)){
  print(gff.names2[p])
  gffContent <- as.matrix(read.delim(gff.files2[p], sep='\t',quote="",stringsAsFactors = FALSE, fill=TRUE)) 
  
  ##Create blank vector to populate with annotation
  blankv <- vector(mode="character", length=dim(gffContent)[1])
  
  #Create a matrix to store elements for gene name
  namesMat <- matrix(NA, ncol=4,nrow=dim(gffContent)[1])
  for (l in 1:dim(gffContent)[1]){
    #For old gbff file names w/o assembly version included
    #namesMat[l,1] <- sub("^([^.]*).*", "\\1",gff.names2[p])
    #Remove .gff from file
    namesMat[l,1] <- substr(gff.names2[p],1,(nchar(gff.names2[p])-4))
    namesMat[l,2] <- gffContent[l,1]
    namesMat[l,3] <- as.numeric(gffContent[l,4])
    namesMat[l,4] <- as.numeric(gffContent[l,5])
  }
  
  #Fill blank vector with new gene name
  for (m in 1:dim(gffContent)[1]){
    blankv[m] <- paste(namesMat[m,1],"--",namesMat[m,2],"--",namesMat[m,3],"--", namesMat[m,4], sep="")
  }
  
  #Append the new vector to old GFF on column 11 in new matrix newGFFwID
  newGFFwID <- cbind(gffContent,blankv)
  #Fix name of file to remove assembly version
  newName <- sub("^([^.]*).*", "\\1",gff.names2[p])
  #Write out new file with ID appended
  write.table(newGFFwID, paste(WD,"/GFFwID/",newName,".gff", sep=""),sep='\t',quote=FALSE,row.names=FALSE,col.names=FALSE)
}
###write.table(newGFFwID, paste(sub("^([^.]*).*", "\\1",gff.names2[p])".gff", sep=""),sep='\t',quote=FALSE,row.names=FALSE,col.names=FALSE)
###END STEP 1


##STEP 2 - REFORMAT THE OUTPUT OF MCL 
#READ THE OUTPUT OF MCL - SPECIFY DIRECTORY AND FILE

print("Appending and reformatting cluster numbers.....")

setwd(WD)
MCLdir <- paste(WD, "/MCL_Clusters", sep = "", collapse = NULL)
MCL_out <- paste(MCLdir, "/out.FILE_ALL_BLAST_filtered.mci.I25", sep = "", collapse = NULL)
rawMCL <- read.table(MCL_out, fill=TRUE, na.strings="NA")

#MAKE a two column matrix from datamat with gene and clusterID
geneClust <- matrix(NA, ncol=2, nrow=length(which(rawMCL!="")))

#Start to fill the geneClust Matrix by scanning each element and assigning its line number
#initialize counter
a <- 1
#Iterate over each line
for (i in 1:(dim(rawMCL))[1]){
  #print(i/(dim(rawMCL)[1])*100)
  #Read each element in the line; first find the number of non-empty elements
  for (b in 1:length(which(rawMCL[i,]!=""))){
    
    #Write each line to the geneClust Matrix
    geneClust[a,1] <- as.character(rawMCL[i,b])
    geneClust[a,2] <- i
    ##update counter
    a <- a+1
  }
}

##WRITE THE NEW FILE FOR POSTERITY AND IF NECESSARY REUSE W/OUT RECALC

Cluster <- paste(WD,"/RS_Cluster_ID", "/Cluster_ID_Assign.txt", sep = "", collapse = NULL)
write.table(geneClust, Cluster ,sep='\t',quote=FALSE,row.names=FALSE,col.names=FALSE)

###END STEP 2
###CHECK THAT IT WORKED
###CHECK THAT IT WORKED



############### ALL EDITED, RUN TO CHECK #########

print("Creating Homology Cluster File for JCE..........")

##STEP 3 - GENERATE THE CLUSTER FILE FOR JCE
gffwID.files <- list.files(GFFwID, full=TRUE)
rawMCL <- read.table(Cluster, fill=TRUE, na.strings="NA")

##MAKE SURE THIS FILE ISNT IN THE SAME DIRECTORY AS THE GFFs


RSClusterIDAssign<- paste(WD,"/RS_Cluster_ID","/Cluster_ID_Assign.txt" , sep = "", collapse = NULL)
geneClust2 <- read.table(RSClusterIDAssign, colClasses=c("character","character"))

#AT THE MOMENT READ IN FILE FOR CONVIENIENCE OF GRABBING GENOME NAMES - IF FINE NAMED APPROPRIATELY CAN WORK
gff.names3 <- list.files(GFF,full=FALSE)

###mclapply method###

##create core mapping function to feed into mcapply
gff2clusterMapping <- function(GFF.file){
  
  gff <- read.delim(GFF.file,sep='\t',stringsAsFactors = FALSE, quote="")
  match2clust <- unlist(map(mclapply(1:nrow(gff), function(x) which(gff[x,11]==geneClust2[,1]),mc.cores=2),1))
  
  matched.geneClust2 <- geneClust2[match2clust,]
  
  match2gff <- unlist(map(mclapply(1:nrow(matched.geneClust2), function(x) which(matched.geneClust2[x,1]==gff[,11]),mc.cores=cores),1))
  
  temp.outMat <- matrix(NA,ncol=5,nrow=length(match2clust))
  
  temp.outMat[,1] <- sub("^([^.]*).*", "\\1", matched.geneClust2[,1])
  temp.outMat[,2] <- gff[match2gff,1]
  temp.outMat[,3] <- gff[match2gff,4]
  temp.outMat[,4] <- gff[match2gff,5]
  #assign the cluster number by 
  temp.outMat[,5] <- matched.geneClust2[,2]
  
  return(temp.outMat)
}
##call mapping function through mcapply using defined number of cores
final.outMat <- do.call(rbind,mclapply(1:length(gffwID.files), function(x) gff2clusterMapping(gffwID.files[x]),mc.cores = cores))
final.outMat <- final.outMat[order(as.numeric(final.outMat[,5])),]

setwd(WD)
write.table(final.outMat,"ClusterID.txt", sep='\t',quote=FALSE,row.names=FALSE,col.names=FALSE)
###Nested for loop method###
##these should be variablized for parallelization depending on cores available CAN CHANGE THESE BY HAND
## instruction on how to do two or three files at a time: replace line below with: for(g in 1:2){    or    for(g in 3:4){

#CREATE A NEW MATRIX TO HOLD DATA 
##outMat <- matrix(NA, ncol=5, nrow=length(which(geneClust2!="")))

##for(g in 1:length(gff.names3)){
##  print(g/length(gff.names3)*100)

## revert to gff
##  gff <- read.table(gffwID.files[g],sep='\t',colClasses=c("character","character","character", "numeric", "numeric", "character", "numeric", "character", "character", "numeric","character"), quote="")
##  print("Matching......")


##  for(h in 1:nrow(gff)){
##    #Match the gene name to the line containing it in GeneCluster to get line assignment
##    row.num <- grep(gff[h,11],geneClust2[,1])

##    #grab the file name with regex, replace with capture group #1
##    #outMat[row.num,1] <- sub("(\w*_\w+_\w+[^\  \s])", "\\1",gff[h,])
##    outMat[row.num,1] <- sub("^([^.]*).*", "\\1", gff.names3[g])
#fill in new columns in outMat from gff file columns
##    outMat[row.num,2] <- gff[h,1]
##    outMat[row.num,3] <- gff[h,4]
##    outMat[row.num,4] <- gff[h,5]
##    #assign the cluster number by 
##    outMat[row.num,5] <- as.numeric(geneClust2[row.num,2])
##  }
##}

##Create a new matrix with no NA rows
##exportMat <- outMat[rowSums(is.na(outMat))!=5,]

###IMPORTANT TO MODIFY HERE IF SPLITTING JOB
##Write a file for JCE Change X into range of files specified above
##write.table(final.outMat,"ClusterID_X.txt", sep='\t',quote=FALSE,row.names=FALSE,col.names=FALSE)

###END STEP 3
##IF the jobs in Step 3 were split it is now time to gather all of the output and cat the files together into a master file
print("Post processing complete. You can find your files in the 'OUTPUT_FOR_JCE' directory.")
quit(save="no",status=0,runLast=TRUE)
